''' Create the finite element mesh and element sets.'''
import math
import xc_base
import geom
import xc

from model import predefined_spaces
from materials.aci import ACI_materials
from materials import typical_materials
from materials.sections import section_properties
from materials.astm_aisc import ASTM_materials
from postprocess import output_handler
from actions import load_cases as lcm
from actions import combinations as combs
from postprocess.config import default_config


from funcs import create_lines, apply_nonPrismatic_section


FEcase= xc.FEProblem()
FEcase.title= 'Industrial structure zeinali'

preprocessor= FEcase.getPreprocessor
nodes= preprocessor.getNodeHandler
elements= preprocessor.getElementHandler
points= preprocessor.getMultiBlockTopology.getPoints
lines= preprocessor.getMultiBlockTopology.getLines


# groups= preprocessor.getSets
h_col = 7.6
dy = 2.5
slop = .2
span = 25
n = 0
rafter_percent = .25

dx = rafter_percent * span
x1 = 0
x2 = dx
x3 = .5 * span
x4 = span - dx
x5 = span
x6 = span + dx
x7 = 1.5 * span
x8 = 2 * span - dx
x9 = 2 * span

z1 = 0
z2 = h_col
z3 = h_col + dx * slop
z4 = h_col + .5 * span * slop

ndiv = 10 # number of diviation for column

pt0= points.newPntIDPos3d(n,geom.Pos3d(x1, 0, z1)); n += 1
pt1= points.newPntIDPos3d(n,geom.Pos3d(x5, 0, z1)); n += 1
pt2= points.newPntIDPos3d(n,geom.Pos3d(x9, 0, z1)); n += 1
for pt in (pt0, pt1, pt2):
    pt.setProp("labels",['xc_supports'])
pt3= points.newPntIDPos3d(n,geom.Pos3d(x1, 0, z2)); n += 1
pt4= points.newPntIDPos3d(n,geom.Pos3d(x2, 0, z3)); n += 1
pt5= points.newPntIDPos3d(n,geom.Pos3d(x3, 0, z4)); n += 1
pt6= points.newPntIDPos3d(n,geom.Pos3d(x4, 0, z3)); n += 1
pt7= points.newPntIDPos3d(n,geom.Pos3d(x5, 0, z2)); n += 1
pt8= points.newPntIDPos3d(n,geom.Pos3d(x6, 0, z3)); n += 1
pt9= points.newPntIDPos3d(n,geom.Pos3d(x7, 0, z4)); n += 1
pt10= points.newPntIDPos3d(n,geom.Pos3d(x8, 0, z3)); n += 1
pt11= points.newPntIDPos3d(n,geom.Pos3d(x9, 0, z2)); n += 1

col_frame_a, n = create_lines(pt0, pt3, 10, points,lines, n)
col_frame_c, n = create_lines(pt1, pt7, 10, points,lines, n)
col_frame_e, n = create_lines(pt2, pt11, 10, points,lines, n)

# l1= lines.newLine(0 , 3); l1.setProp("labels",['xc_columns', 'frame_a'])
l2= lines.newLine(3 , 4); l2.setProp("labels",['xc_beams', 'beam1'])
l3= lines.newLine(4 , 5); l3.setProp("labels",['xc_beams', 'beam2'])
l4= lines.newLine(5 , 6); l4.setProp("labels",['xc_beams', 'beam2'])
l5= lines.newLine(6 , 7); l5.setProp("labels",['xc_beams', 'beam1'])
# l6= lines.newLine(1 , 7); l6.setProp("labels",['xc_columns', 'frame_c'])
l7= lines.newLine(7 , 8); l7.setProp("labels",['xc_beams', 'beam1'])
l8= lines.newLine(8 , 9); l8.setProp("labels",['xc_beams', 'beam2'])
l9= lines.newLine(9 , 10); l9.setProp("labels",['xc_beams', 'beam2'])
l10= lines.newLine(10 , 11); l10.setProp("labels",['xc_beams', 'beam1'])
# l11= lines.newLine(2 , 11); l11.setProp("labels",['xc_columns', 'frame_e'])
for l in col_frame_a:
    l.setProp("labels", ['xc_columns', 'xc_col_a'])
for l in col_frame_c:
    l.setProp("labels", ['xc_columns', 'xc_col_c'])
for l in col_frame_e:
    l.setProp("labels", ['xc_columns', 'xc_col_e'])

xcTotalSet= preprocessor.getSets.getSet('total')

preprocessor=FEcase.getPreprocessor
nodes= preprocessor.getNodeHandler
modelSpace= predefined_spaces.StructuralMechanics3D(nodes) 


beam1Set= preprocessor.getSets.defSet('beam1Set')
beam2Set= preprocessor.getSets.defSet('beam2Set')
supportSet= preprocessor.getSets.defSet('supportSet')
col_frame_a_set = preprocessor.getSets.defSet('colFrameASet')
col_frame_c_set = preprocessor.getSets.defSet('colFrameCSet')
col_frame_e_set = preprocessor.getSets.defSet('colFrameESet')

setsFromLabels= {
        'xc_col_a':col_frame_a_set,
        'xc_col_c':col_frame_c_set,
        'xc_col_e':col_frame_e_set,
        'xc_supports': supportSet,
        'beam1': beam1Set,
        'beam2': beam2Set,
        }

for l in xcTotalSet.getLines:
    if(l.hasProp('labels')):
        labels= l.getProp('labels')
        for key in setsFromLabels:
            if(labels.count(key)>0):
                xcSet= setsFromLabels[key]
                xcSet.getLines.append(l)



## Supports
for p in xcTotalSet.getPoints:
    if(p.hasProp('labels')):
        labels= p.getProp('labels')
        if(labels.count('xc_supports')>0):
            supportSet.getPoints.append(p)
        else:
            n= p.getNode()
            modelSpace.fixNode('F0F_0F0',n.tag)


# Materials
## Steel material
steel_W= ASTM_materials.A992   #steel support W shapes
steel_W.gammaM= 1.00
## Profile geometry
outer_column_profile= ASTM_materials.WShape(steel_W,'W18X143')
inner_column_profile= outer_column_profile

beam1_profile= outer_column_profile
beam2_profile= outer_column_profile

# Mesh generation.
coordTransf= preprocessor.getTransfCooHandler.newCorotCrdTransf3d('coordTransf')
coordTransf.xzVector= xc.Vector([1.0,0,0])

seedElemHandler= preprocessor.getElementHandler.seedElemHandler
seedElemHandler.defaultTransformation= 'coordTransf'

## Columns.
# modelSpace.createElasticBeams(columnSet, outer_column_profile, coordTransf, xc.Vector([0,1,0]), nDiv= 6)
bf_col_ae = 20
tf_col_ae = 1.2
tw_col_ae = .8
hw_col_ae_bot = 20
hw_col_ae_top = 70

apply_nonPrismatic_section([col_frame_a_set, col_frame_e_set, col_frame_c_set], bf_col_ae, \
                           tf_col_ae, tw_col_ae, hw_col_ae_bot, hw_col_ae_top, coordTransf, \
                           modelSpace, steel_W, preprocessor)

## Main beams.
modelSpace.createElasticBeams(beam1Set, beam1_profile, coordTransf, xc.Vector([0,1.0,0]), nDiv= 6)
modelSpace.createElasticBeams(beam2Set, beam2_profile, coordTransf, xc.Vector([0,1.0,0]), nDiv= 6)

# Constraints
## Columns
for p in supportSet.getPoints:
    if(not p.hasNode):
        lmsg.warning('point: '+str(p)+' not meshed.')
    n= p.getNode()
    modelSpace.fixNode('000_0F0',n.tag)



# Set definition
# diagonalSet= modelSpace.setSum('diagonalSet',[diagonalSetA, diagonalSetB, diagonalSet1, diagonalSet2, hDiagonalSet])
# columnSet= modelSpace.setSum('columnSet',[lowerColumnSet, intermediateColumnSet, upperColumnSet, topColumnSet])
# columnSet.fillDownwards()
# barSet= modelSpace.setSum('barSet',[columnSet,xLowerBeamSet, yLowerBeamSet, xBeamSet,yBeamSet,stringerBeamSet,stairBeamSet])

# # Member sets.
# mainMemberSet= modelSpace.setSum('mainMemberSet',[barSet])
# secondaryMemberSet= modelSpace.setSum('secondaryMemberSet',[diagonalSet])
allMemberSet= modelSpace.setSum('allMemberSet', [beam1Set, beam2Set, col_frame_a_set, col_frame_e_set, col_frame_c_set])


# columnSet.description= 'Columns'
# xLowerBeamSet.description= 'Beams on frames A and B, levels 1 to 3'
# yLowerBeamSet.description= 'Beams on frames 1 and 2, levels 1 to 3'
# xBeamSet.description= 'Beams on frames A and B, levels 4 and up'
# yBeamSet.description= 'Beams on frames 1 and 2, levels 4 and up'
# diagonalSetA.description= 'Bracing on frame A'
# diagonalSetB.description= 'Bracing on frame B'
# diagonalSet1.description= 'Bracing on frame 1'
# diagonalSet2.description= 'Bracing on frame 2'
# hDiagonalSet.description= 'Horizontal bracing (level 4)'
# stairBeamSet.description= 'Stair landing beams'
# stringerBeamSet.description= 'Stringer beams'
# Graphic stuff.
oh= output_handler.OutputHandler(modelSpace)

oh.displayBlocks()#setToDisplay= stringerBeamSet)
oh.displayFEMesh()
oh.displayLocalAxes()
# oh.displayLoads()
#oh.displayReactions()
#oh.displayDispRot(itemToDisp='uX')
#oh.displayDispRot(itemToDisp='uY')
#oh.displayDispRot(itemToDisp='uZ')
